@@ -43,6 +43,8 @@
 #include <openssl/err.h>
 #endif
 
+#include <syslog.h>
+
 /* undef this if you have problems with non blocking accept() */
 #define NONE_BLOCK
 
@@ -69,6 +71,9 @@
 /* status file name */
 #define STATUS_FILE_NAME "pgpool_status"
 
+/* default string used to identify pgpool on syslog output */
+#define DEFAULT_SYSLOG_IDENT "pgpool"
+
 typedef enum {
 	POOL_CONTINUE = 0,
 	POOL_IDLE,
@@ -114,6 +119,7 @@ typedef struct
 	int minor;	/* protocol minor version */
 	char *database;	/* database name in startup_packet (malloced area) */
 	char *user;	/* user name in startup_packet (malloced area) */
+	char *application_name;		/* not malloced are. pointing to in startup_packet */
 } StartupPacket;
 
 typedef struct CancelPacket
@@ -261,12 +267,14 @@ typedef struct {
  */
 extern bool pool_is_node_to_be_sent_in_current_query(int node_id);
 extern int pool_virtual_master_db_node_id(void);
+extern BACKEND_STATUS* my_backend_status[];
+extern int my_master_node_id;
 
 #define VALID_BACKEND(backend_id) \
-	((RAW_MODE && (backend_id) == REAL_MASTER_NODE_ID) || \
-	 (pool_is_node_to_be_sent_in_current_query((backend_id)) && \
-	  ((BACKEND_INFO((backend_id)).backend_status == CON_UP) || \
-	   (BACKEND_INFO((backend_id)).backend_status == CON_CONNECT_WAIT))))
+	((RAW_MODE && (backend_id) == REAL_MASTER_NODE_ID) ||		\
+	(pool_is_node_to_be_sent_in_current_query((backend_id)) &&	\
+	 ((*(my_backend_status[(backend_id)]) == CON_UP) ||			\
+	  (*(my_backend_status[(backend_id)]) == CON_CONNECT_WAIT))))
 
 #define CONNECTION_SLOT(p, slot) ((p)->slots[(slot)])
 #define CONNECTION(p, slot) (CONNECTION_SLOT(p, slot)->con)
@@ -344,7 +352,8 @@ typedef enum {
 	NODE_UP_REQUEST = 0,
 	NODE_DOWN_REQUEST,
 	NODE_RECOVERY_REQUEST,
-	CLOSE_IDLE_REQUEST
+	CLOSE_IDLE_REQUEST,
+	PROMOTE_NODE_REQUEST
 } POOL_REQUEST_KIND;
 
 typedef struct {
@@ -353,6 +362,7 @@ typedef struct {
 	int master_node_id;	/* the youngest node id which is not in down status */
 	int primary_node_id;	/* the primary node id in streaming replication mode */
 	int conn_counter;
+	bool switching;	/* it true, failover or failback is in progress */
 } POOL_REQUEST_INFO;
 
 /* description of row. corresponding to RowDescription message */
@@ -399,6 +409,8 @@ extern volatile sig_atomic_t exit_request;
 #define QUERY_STRING_BUFFER_LEN 1024
 extern char query_string_buffer[];		/* last query string sent to simpleQuery() */
 
+extern BACKEND_STATUS private_backend_status[MAX_NUM_BACKENDS];
+
 /*
  * public functions
  */
@@ -443,9 +455,10 @@ extern POOL_STATUS NoticeResponse(POOL_CONNECTION *frontend,
 
 extern void notice_backend_error(int node_id);
 extern void degenerate_backend_set(int *node_id_set, int count);
-extern bool degenerate_backend_set_ex(int *node_id_set, int count, bool test_only);
+extern void promote_backend(int node_id);
 extern void send_failback_request(int node_id);
 
+
 extern void pool_set_timeout(int timeoutval);
 extern int pool_check_fd(POOL_CONNECTION *cp);
 
@@ -535,7 +548,7 @@ extern int pool_clear_cache_by_time(Interval *interval, int size);
 extern POOL_STATUS pool_execute_query_cache_lookup(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend, Node *node);
 
 /* pool_hba.c */
-extern void load_hba(char *hbapath);
+extern int load_hba(char *hbapath);
 extern void ClientAuthentication(POOL_CONNECTION *frontend);
 
 /* pool_ip.c */
@@ -560,6 +573,7 @@ extern int wait_connection_closed(void);
 /* child.c */
 extern void cancel_request(CancelPacket *sp);
 extern void check_stop_request(void);
+extern void pool_initialize_private_backend_status(void);
 
 /* pool_process_query.c */
 extern void reset_variables(void);
